@@ -1,6 +1,8 @@
+import datetime
 import posixpath
+import warnings
 from collections import OrderedDict
-from typing import Any, Dict, Iterable, Optional, Type
+from typing import Any, ClassVar, Dict, Iterable, Optional, Type
 
 import magic
 from django.contrib.contenttypes.fields import GenericForeignKey
@@ -9,36 +11,43 @@ from django.contrib.staticfiles.finders import find
 from django.contrib.staticfiles.storage import staticfiles_storage
 from django.core import checks
 from django.core.files import File
+from django.core.files.storage import Storage
 from django.db import models
 from django.db.models.base import ModelBase
 from django.db.models.fields.files import FieldFile
 from django.db.models.functions import Coalesce
-from django.db.models.utils import make_model_tuple
 from django.template import loader
-from django.utils.module_loading import import_string
 from django.utils.timezone import now
 from django.utils.translation import gettext_lazy as _
 from polymorphic.base import PolymorphicModelBase
 from polymorphic.models import PolymorphicModel
 
-from ..conf import FILE_ICON_DEFAULT, FILE_ICON_OVERRIDES, settings
-from ..helpers import _get_item_types, _set_item_types, build_variations
-from ..storage import upload_storage
+from .. import exceptions
+from ..conf import FILE_ICON_DEFAULT, FILE_ICON_OVERRIDES, IMAGE_ITEM_VARIATIONS, settings
+from ..helpers import (
+    _get_item_types,
+    _set_item_types,
+    build_variations,
+    iterate_variation_names,
+)
+from ..storage import default_storage
 from ..variations import PaperVariation
 from .base import (
     FileFieldResource,
     NoPermissionsMetaBase,
+    Resource,
     ResourceBaseMeta,
     VersatileImageResourceMixin,
 )
 from .fields import CollectionItem
+from .fields.base import DynamicStorageFileField
 from .fields.collection import ContentItemRelation
 from .image import VariationalFileField
-from .mixins import BacklinkModelMixin
-from .utils import generate_filename
+from .mixins import BacklinkModelMixin, EditableResourceMixin
 
 __all__ = [
     "CollectionItemBase",
+    "CollectionFileItemBase",
     "CollectionBase",
     "FilePreviewMixin",
     "FileItemBase",
@@ -62,7 +71,7 @@ class ItemTypesDescriptor:
     def __init__(self, name):
         self.name = name
 
-    def __get__(self, instance, cls=None):
+    def __get__(self, instance, cls=None) -> Dict[str, CollectionItem]:
         if cls is None:
             cls = type(instance)
 
@@ -71,7 +80,7 @@ class ItemTypesDescriptor:
             return OrderedDict(getattr(cls, cache_attname))
 
         item_types = OrderedDict()
-        parents = [base for base in cls.mro() if issubclass(base, CollectionBase)]
+        parents = [base for base in cls.__mro__ if issubclass(base, CollectionBase)]
         for base in reversed(parents):
             base_item_types = _get_item_types(base)
             if base_item_types is not None:
@@ -124,29 +133,80 @@ class CollectionMeta(NoPermissionsMetaBase, ModelBase):
         return new_class
 
 
+class CollectionManager(models.Manager):
+    """
+    Из-за того, что в большинстве случаев коллекции являются прокси-моделями
+    для Collection, любые запросы от имени любой прокси-коллекции будут распространяться
+    на все экземпляры Collection.
+
+    Этот менеджер ограничивает область действия запросов той прокси-коллекцией,
+    от которой исходит запрос.
+    
+    Для не-прокси моделей поведение стандартное: запрос вернёт все экземпляры таблицы.
+    """
+
+    def get_queryset(self):
+        if self.model._meta.proxy:
+            collection_ct = ContentType.objects.get_for_model(
+                self.model, for_concrete_model=False
+            )
+            return super().get_queryset().filter(collection_content_type=collection_ct)
+        else:
+            return super().get_queryset()
+
+
 class CollectionBase(BacklinkModelMixin, metaclass=CollectionMeta):
+    collection_content_type = models.ForeignKey(
+        ContentType,
+        null=True,
+        on_delete=models.SET_NULL,
+        editable=False
+    )
     items = ContentItemRelation(
         "paper_uploads.CollectionItemBase",
         content_type_field="collection_content_type",
         object_id_field="collection_id",
         for_concrete_model=False,
     )
-    created_at = models.DateTimeField(_("created at"), default=now, editable=False)
+    created_at = models.DateTimeField(
+        _("created at"),
+        default=now,
+        editable=False
+    )
+    modified_at = models.DateTimeField(
+        _("changed at"),
+        default=now,
+        editable=False
+    )
+
+    default_mgr = models.Manager()  # fix migrations manager
+    objects = CollectionManager()
 
     class Meta:
         proxy = False
         abstract = True
         default_permissions = ()
+        default_manager_name = "default_mgr"
         verbose_name = _("collection")
         verbose_name_plural = _("collections")
 
+    def __iter__(self):
+        return self.get_items().iterator()
+
+    def save(self, *args, **kwargs):
+        if not self.collection_content_type:
+            self.collection_content_type = ContentType.objects.get_for_model(
+                self, for_concrete_model=False
+            )
+        super().save(*args, **kwargs)
+
     def delete(self, using=None, keep_parents=False):
-        # Удаляем элементы вручную из-за рекурсии (см. clean_uploads.py, строка 145)
+        # Удаляем элементы вручную из-за рекурсии.
         # Удалить элементы с помощью `.all().delete()` нельзя из-за того, что в
         # файле (django/db/models/deletion.py:248) указано следующее:
         #   model = new_objs[0].__class__
         # Это приводит к тому, что все полиморфные модели удаляются через модель
-        # первой модели в спсике.
+        # первого экземпляра в QuerySet.
         for item_type in self.item_types:
             self.get_items(item_type).delete()
 
@@ -162,119 +222,50 @@ class CollectionBase(BacklinkModelMixin, metaclass=CollectionMeta):
     @classmethod
     def get_item_model(cls, item_type: str) -> 'Type[CollectionItemBase]':
         if item_type not in cls.item_types:
-            raise ValueError(_("Unknown item type: %s") % item_type)
+            raise exceptions.InvalidItemType(item_type)
         return cls.item_types[item_type].model
 
-    def set_owner_from(self, field: models.Field):
-        self.owner_app_label, self.owner_model_name = make_model_tuple(field.model)
-        self.owner_fieldname = field.name
-
     def get_items(self, item_type: str = None) -> 'models.QuerySet[CollectionItemBase]':
-        # TODO: что если класс элемента был удален из коллекции, но элементы остались?
         if item_type is None:
             return self.items.order_by("order")
         if item_type not in self.item_types:
-            raise ValueError(_("Unknown item type: %s") % item_type)
-        return self.items.filter(item_type=item_type).order_by("order")
+            raise exceptions.InvalidItemType(item_type)
+        return self.items.filter(type=item_type).order_by("order")
 
-    def detect_item_type(self, *args, **kwargs) -> Optional[str]:
+    def get_last_modified(self) -> datetime.datetime:
         """
-        Генератор, поочередно проверяющий классы элементов коллекции
-        на возможность представления данных, переданных в параметрах.
+        Получение даты модификации коллекции с учётом её элементов.
         """
-        for item_type, field in self.item_types.items():
-            if hasattr(field.model, "file_supported"):
-                if field.model.file_supported(*args, **kwargs):
-                    yield item_type
+        dates = [self.created_at, self.modified_at]
+        item_date = self.items.aggregate(
+            date=models.Max("modified_at")
+        )["date"]
 
+        if item_date is not None:
+            dates.append(item_date)
 
-class CollectionManager(models.Manager):
-    """
-    Из-за того, что все галереи являются прокси-моделями, запросы от имени
-    любого из классов галереи затрагивает абсолютно все объекты галереи.
-    С помощью этого менеджера можно работать только с галереями текущего типа.
-    """
-
-    def get_queryset(self):
-        collection_ct = ContentType.objects.get_for_model(
-            self.model, for_concrete_model=False
-        )
-        return super().get_queryset().filter(collection_content_type=collection_ct)
+        return max(*dates)
 
 
 class Collection(CollectionBase):
-    collection_content_type = models.ForeignKey(
-        ContentType, null=True, on_delete=models.SET_NULL, editable=False
-    )
-
-    default_mgr = models.Manager()  # fix migrations manager
-    objects = CollectionManager()
+    VARIATIONS: ClassVar[dict]
 
     class Meta:
         proxy = False  # явно указываем, что это не прокси-модель
-        default_manager_name = "default_mgr"
-
-    def save(self, *args, **kwargs):
-        if not self.collection_content_type:
-            self.collection_content_type = ContentType.objects.get_for_model(
-                self, for_concrete_model=False
-            )
-        super().save(*args, **kwargs)
 
 
 # ======================================================================================
 
 
 class CollectionItemMetaBase(PolymorphicModelBase, ResourceBaseMeta):
-    """
-    Приём, позволяющий переопределить OneToOne-связь между моделями при наследовании
-    от абстрактной модели.
-
-    По умолчанию, при наследовании от абстрактой модели, унаследованной от конкретной
-    (concrete), попытка переопределния OneToOne-связи не замещает поле по умолчанию,
-    а добавляет второе.
-
-    Ссылка:
-    https://docs.djangoproject.com/en/3.2/topics/db/models/#specifying-the-parent-link-field
-    """
-    def __new__(cls, name, bases, attrs, **kwargs):
-        parents = [b for b in bases if isinstance(b, ModelBase)]
-
-        parent_links = {}
-        for base in reversed(parents):
-            # Conceptually equivalent to `if base is Model`.
-            if not hasattr(base, '_meta'):
-                continue
-
-            # Locate OneToOneField instances.
-            for field in base._meta.local_fields:
-                if isinstance(field, models.OneToOneField) and field.remote_field.parent_link:
-                    key = make_model_tuple(field.remote_field.model)
-                    parent_links.setdefault(key, []).append(field)
-
-        for fieldname, field in list(attrs.items()):
-            if isinstance(field, models.OneToOneField) and field.remote_field.parent_link:
-                key = make_model_tuple(field.remote_field.model)
-                if key in parent_links:
-                    inherited_field = parent_links[key].pop()
-                    if fieldname != inherited_field.name:
-                        # force delete inherited field
-                        attrs[inherited_field.name] = None
-
-        return super().__new__(cls, name, bases, attrs)
+    pass
 
 
-class CollectionItemBase(PolymorphicModel, metaclass=CollectionItemMetaBase):
+class CollectionItemBase(EditableResourceMixin, PolymorphicModel, Resource, metaclass=CollectionItemMetaBase):
     """
     Базовый класс элемента коллекции.
     """
 
-    # Флаг для индикации базового класса элемента коллекции.
-    # См. метод _check_form_class()
-    __BaseCollectionItem = True
-
-    change_form_class: Optional[str] = None
-
     # путь к шаблону, представляющему элемент коллекции в админке
     template_name: Optional[str] = None
 
@@ -289,12 +280,16 @@ class CollectionItemBase(PolymorphicModel, metaclass=CollectionItemMetaBase):
         for_concrete_model=False,
     )
 
-    item_type = models.CharField(
+    type = models.CharField(
         _("type"), max_length=32, db_index=True, editable=False
     )
     order = models.IntegerField(_("order"), default=0, editable=False)
 
     class Meta:
+        # Используется обратный порядок полей в составном индексе,
+        # т.к. слективность поля collection_id выше, а для поля
+        # `collection_content_type` уже есть отдельный индекс.
+        index_together = [("collection_id", "collection_content_type")]
         verbose_name = _("item")
         verbose_name_plural = _("items")
 
@@ -302,43 +297,12 @@ class CollectionItemBase(PolymorphicModel, metaclass=CollectionItemMetaBase):
     def check(cls, **kwargs):
         return [
             *super().check(**kwargs),
-            *cls._check_form_class(),
             *cls._check_template_name(),
         ]
 
-    @classmethod
-    def _check_form_class(cls, **kwargs):
-        flag = "_{}__BaseCollectionItem".format(cls.__name__)
-        if getattr(cls, flag, None) is True or cls._meta.abstract:
-            return []
-
-        errors = []
-        if cls.change_form_class is None:
-            errors.append(
-                checks.Error(
-                    "{} requires a definition of 'change_form_class'".format(
-                        cls.__name__
-                    ),
-                    obj=cls,
-                )
-            )
-        else:
-            try:
-                import_string(cls.change_form_class)
-            except ImportError:
-                errors.append(
-                    checks.Error(
-                        "The value of 'change_form_class' refers to '%s', which does "
-                        "not exists" % cls.change_form_class,
-                        obj=cls,
-                    )
-                )
-        return errors
-
     @classmethod
     def _check_template_name(cls, **kwargs):
-        flag = "_{}__BaseCollectionItem".format(cls.__name__)
-        if getattr(cls, flag, None) is True or cls._meta.abstract:
+        if cls._meta.abstract or cls is CollectionItemBase:
             return []
 
         errors = []
@@ -351,6 +315,24 @@ class CollectionItemBase(PolymorphicModel, metaclass=CollectionItemMetaBase):
             )
         return errors
 
+    @property
+    def item_type(self):
+        warnings.warn(
+            "'item_type' is deprecated in favor of 'type'",
+            DeprecationWarning,
+            stacklevel=2
+        )
+        return self.type
+
+    @item_type.setter
+    def item_type(self, value: str):
+        warnings.warn(
+            "'item_type' is deprecated in favor of 'type'",
+            DeprecationWarning,
+            stacklevel=2
+        )
+        self.type = value
+
     def get_order(self):
         max_order = CollectionItemBase.objects.filter(
             collection_content_type_id=self.collection_content_type_id,
@@ -362,21 +344,28 @@ class CollectionItemBase(PolymorphicModel, metaclass=CollectionItemMetaBase):
         return 0 if max_order is None else max_order + 1
 
     def save(self, *args, **kwargs):
-        if not self.pk:
-            # Попытка решить проблему того, что при создании коллекции,
-            # элементы отсортированы в порядке загрузки, а не в порядке
-            # добавления. Код ниже не решает проблему полностью,
-            # но уменьшает её влияние.
+        if not self.pk and self.collection_id:
+            # Попытка решить проблему того, что при создании коллекции элементы
+            # отсортированы в порядке загрузки, а не в порядке добавления.
+            # Код ниже не решает проблему полностью, но уменьшает её влияние.
             if self.order is None or self.collection.items.filter(order=self.order).exists():
                 self.order = self.get_order()
         super().save(*args, **kwargs)
 
+    @classmethod
+    def accept(cls, *args, **kwargs) -> bool:
+        """
+        Возвращает True, если переданные данные могут быть представлены
+        текущей моделью элемента коллекции.
+        """
+        raise NotImplementedError
+
     def as_dict(self) -> Dict[str, Any]:
         return {
             **super().as_dict(),
             "collectionId": self.collection_id,
-            "itemType": self.item_type,
-            "caption": self.get_caption(),
+            "itemType": self.type,  # TODO: deprecated
+            "type": self.type,
             "order": self.order,
             "preview": self.render_preview(),
         }
@@ -384,13 +373,21 @@ class CollectionItemBase(PolymorphicModel, metaclass=CollectionItemMetaBase):
     def get_collection_class(self) -> Type[CollectionBase]:
         return self.collection_content_type.model_class()
 
-    def get_itemtype_field(self) -> Optional[CollectionItem]:
+    def get_item_type_field(self) -> Optional[CollectionItem]:
         collection_cls = self.get_collection_class()
         for name, field in collection_cls.item_types.items():
-            if field.model is type(self):
+            if field.model is type(self) or (field.model._meta.proxy and field.model._meta.concrete_model is type(self)):
                 return field
         return None
 
+    def get_itemtype_field(self) -> Optional[CollectionItem]:
+        warnings.warn(
+            "'get_itemtype_field' is deprecated in favor of 'get_item_type_field'",
+            DeprecationWarning,
+            stacklevel=2
+        )
+        return self.get_item_type_field()
+
     def attach_to(self, collection: CollectionBase):
         """
         Подключение элемента к коллекции.
@@ -401,15 +398,11 @@ class CollectionItemBase(PolymorphicModel, metaclass=CollectionItemMetaBase):
         self.collection_id = collection.pk
         for name, field in collection.item_types.items():
             if field.model is type(self):
-                self.item_type = name
+                self.type = name
                 break
         else:
             raise TypeError(_("Unsupported collection item: %s") % type(self).__name__)
 
-    def get_caption(self):
-        """ Заголовок для виджета в админке """
-        return self.get_basename()
-
     def render_preview(self):
         """ Отображение элемента коллекции в админке """
         context = self.get_preview_context()
@@ -419,11 +412,11 @@ class CollectionItemBase(PolymorphicModel, metaclass=CollectionItemMetaBase):
         return {
             "item": self,
             "width": settings.COLLECTION_ITEM_PREVIEW_WIDTH,
-            "height": settings.COLLECTION_ITEM_PREVIEW_HEIGTH,
+            "height": settings.COLLECTION_ITEM_PREVIEW_HEIGHT,
         }
 
 
-class CollectionFileItemBase(CollectionItemBase, FileFieldResource, metaclass=CollectionItemMetaBase):
+class CollectionFileItemBase(CollectionItemBase, FileFieldResource):
     """
     Базовый класс элемента галереи, содержащего файл.
     """
@@ -432,13 +425,25 @@ class CollectionFileItemBase(CollectionItemBase, FileFieldResource, metaclass=Co
         abstract = True
 
     @classmethod
-    def file_supported(cls, file: File) -> bool:
-        """
-        Проверка возможности представления загруженного файла
-        текущим классом элемента в коллекции.
-        """
+    def accept(cls, file: File) -> bool:
         raise NotImplementedError
 
+    def get_file_storage(self) -> Storage:
+        item_type_field = self.get_item_type_field()
+        storage = item_type_field.options.get("storage") or default_storage
+        if callable(storage):
+            storage = storage()
+        return storage
+
+    @classmethod
+    def file_supported(cls, file: File) -> bool:
+        warnings.warn(
+            "file_supported() is deprecated in favor of accept()",
+            DeprecationWarning,
+            stacklevel=2
+        )
+        return cls.accept(file)
+
 
 class FilePreviewMixin(models.Model):
     """
@@ -467,15 +472,13 @@ class FilePreviewMixin(models.Model):
 
 
 class FileItemBase(FilePreviewMixin, CollectionFileItemBase):
-    change_form_class = "paper_uploads.forms.dialogs.collection.FileItemDialog"
+    change_form_class = "paper_uploads.forms.dialogs.collection.ChangeFileItemDialog"
     template_name = "paper_uploads/items/file.html"
     preview_template_name = "paper_uploads/items/preview/file.html"
 
-    file = models.FileField(
+    file = DynamicStorageFileField(
         _("file"),
         max_length=255,
-        storage=upload_storage,
-        upload_to=generate_filename,
     )
     display_name = models.CharField(_("display name"), max_length=255, blank=True)
 
@@ -490,32 +493,34 @@ class FileItemBase(FilePreviewMixin, CollectionFileItemBase):
         super().save(*args, **kwargs)
 
     def get_file_folder(self) -> str:
-        return settings.COLLECTION_FILES_UPLOAD_TO
+        item_type_field = self.get_item_type_field()
+        return item_type_field.options.get("upload_to") or settings.COLLECTION_FILES_UPLOAD_TO
 
     def get_file(self) -> FieldFile:
         return self.file
 
-    def set_file(self, value):
-        self.file = value
-
     def get_file_field(self) -> models.FileField:
         return self._meta.get_field("file")
 
+    def get_caption(self):
+        name = self.display_name or self.basename
+        if self.extension:
+            return "{}.{}".format(name, self.extension)
+        return name
+
     @classmethod
-    def file_supported(cls, file: File) -> bool:
+    def accept(cls, file: File) -> bool:
         return True
 
 
 class SVGItemBase(CollectionFileItemBase):
-    change_form_class = "paper_uploads.forms.dialogs.collection.FileItemDialog"
+    change_form_class = "paper_uploads.forms.dialogs.collection.ChangeFileItemDialog"
     template_name = "paper_uploads/items/svg.html"
     preview_template_name = "paper_uploads/items/preview/svg.html"
 
-    file = models.FileField(
+    file = DynamicStorageFileField(
         _("file"),
         max_length=255,
-        storage=upload_storage,
-        upload_to=generate_filename,
     )
     display_name = models.CharField(_("display name"), max_length=255, blank=True)
 
@@ -530,34 +535,45 @@ class SVGItemBase(CollectionFileItemBase):
         super().save(*args, **kwargs)
 
     def get_file_folder(self) -> str:
-        return settings.COLLECTION_FILES_UPLOAD_TO
+        item_type_field = self.get_item_type_field()
+        return item_type_field.options.get("upload_to") or settings.COLLECTION_FILES_UPLOAD_TO
 
     def get_file(self) -> FieldFile:
         return self.file
 
-    def set_file(self, value):
-        self.file = value
-
     def get_file_field(self) -> models.FileField:
         return self._meta.get_field("file")
 
+    def get_caption(self):
+        name = self.display_name or self.basename
+        if self.extension:
+            return "{}.{}".format(name, self.extension)
+        return name
+
     @classmethod
-    def file_supported(cls, file: File) -> bool:
+    def accept(cls, file: File) -> bool:
         filename, ext = posixpath.splitext(file.name)
         return ext.lower() == ".svg"
 
+    def _prepare_file(self, file: File, **options) -> File:
+        mimetype = magic.from_buffer(file.read(1024), mime=True)
+        file.seek(0)
+        if mimetype != "image/svg+xml":
+            raise exceptions.UnsupportedResource(
+                _("File `%s` is not an svg image") % file.name
+            )
+        return super()._prepare_file(file, **options)
+
 
 class ImageItemBase(VersatileImageResourceMixin, CollectionFileItemBase):
-    PREVIEW_VARIATIONS = settings.COLLECTION_IMAGE_ITEM_PREVIEW_VARIATIONS
-    change_form_class = "paper_uploads.forms.dialogs.collection.ImageItemDialog"
+    PREVIEW_VARIATIONS = IMAGE_ITEM_VARIATIONS
+    change_form_class = "paper_uploads.forms.dialogs.collection.ChangeImageItemDialog"
     template_name = "paper_uploads/items/image.html"
     preview_template_name = "paper_uploads/items/preview/image.html"
 
     file = VariationalFileField(
         _("file"),
         max_length=255,
-        storage=upload_storage,
-        upload_to=generate_filename,
     )
 
     class Meta(CollectionItemBase.Meta):
@@ -565,15 +581,22 @@ class ImageItemBase(VersatileImageResourceMixin, CollectionFileItemBase):
         verbose_name = _("Image item")
         verbose_name_plural = _("Image items")
 
+    def _setup_variation_files(self):
+        # Предотвращение вероятной бесконечной рекурсии.
+        # Ошибка возникает в случае, когда модель коллекции была удалена,
+        # но соответствующий ContentType остался.
+        if "file" not in self.__dict__:
+            return
+
+        super()._setup_variation_files()
+
     def get_file_folder(self) -> str:
-        return settings.COLLECTION_IMAGES_UPLOAD_TO
+        item_type_field = self.get_item_type_field()
+        return item_type_field.options.get("upload_to") or settings.COLLECTION_IMAGES_UPLOAD_TO
 
     def get_file(self) -> FieldFile:
         return self.file
 
-    def set_file(self, value):
-        self.file = value
-
     def get_file_field(self) -> VariationalFileField:
         return self._meta.get_field("file")
 
@@ -581,11 +604,14 @@ class ImageItemBase(VersatileImageResourceMixin, CollectionFileItemBase):
         """
         Превью для админки режутся сразу, а остальное — потом.
         """
-        preview_variations = tuple(self.PREVIEW_VARIATIONS.keys())
+        preview_variations = tuple(iterate_variation_names(self.PREVIEW_VARIATIONS))
         self.recut(names=preview_variations)
 
-        other_variations = tuple(set(names).difference(preview_variations))
-        super().recut_async(other_variations)
+        if not names:
+            names = self.get_variations().keys()
+
+        rest_variations = tuple(set(names).difference(preview_variations))
+        super().recut_async(rest_variations)
 
     def get_variations(self) -> Dict[str, PaperVariation]:
         """
@@ -597,28 +623,28 @@ class ImageItemBase(VersatileImageResourceMixin, CollectionFileItemBase):
         """
         if not hasattr(self, "_variations_cache"):
             collection_cls = self.get_collection_class()
-            itemtype_field = self.get_itemtype_field()
-            variation_config = self.get_variation_config(itemtype_field, collection_cls)
+            item_type_field = self.get_item_type_field()
+            variation_config = self.get_variation_config(collection_cls, item_type_field)
             self._variations_cache = build_variations(variation_config)
         return self._variations_cache
 
     @classmethod
-    def file_supported(cls, file: File) -> bool:
+    def accept(cls, file: File) -> bool:
         mimetype = magic.from_buffer(file.read(1024), mime=True)
-        file.seek(0)  # correct file position after mimetype detection
+        file.seek(0)  # reset file position after mimetype detection
         basetype, subtype = mimetype.split("/", 1)
         return basetype == "image"
 
     @classmethod
     def get_variation_config(
-        cls, rel: Optional[CollectionItem], collection_cls: Type[CollectionBase]
+        cls, collection_cls: Type[CollectionBase], item_type_field: Optional[CollectionItem],
     ) -> Dict[str, Any]:
-        if rel is not None and "variations" in rel.options:
-            variations = rel.options["variations"]
+        if item_type_field is not None and "variations" in item_type_field.options:
+            variations = item_type_field.options["variations"]
         else:
             variations = getattr(collection_cls, "VARIATIONS", None)
-        variations = (variations or {}).copy()
-        variations.update(cls.PREVIEW_VARIATIONS)
+        variations = variations or {}
+        variations = dict(cls.PREVIEW_VARIATIONS, **variations)
         return variations
 
 
@@ -641,7 +667,6 @@ class ImageCollection(Collection):
     """
     Коллекция, позволяющая хранить только изображения.
     """
-
     image = CollectionItem(ImageItem)
 
     @classmethod
